"""
A Requests Session with a base URL.
"""
from collections import OrderedDict
from copy import deepcopy
from typing import Any, Optional, Dict

import requests
from requests.adapters import HTTPAdapter
from requests.cookies import cookiejar_from_dict
from requests.hooks import default_hooks
from requests.models import DEFAULT_REDIRECT_LIMIT
from requests.utils import default_headers
from requests.auth import AuthBase


class DefaultSession(requests.Session):
    """
    A Requests Session with a URL that all requests will use as a base.

    Let's start by looking at an example:
    .. code-block:: python
        # >>> from requests_base import DefaultSession
        >>> session = DefaultSession(base_url='https://example.com/resource/', headers={'api-key': 'aabbcc'})
        >>> r = session.get('sub-resource/', params={'foo': 'bar'})
        >>> print(r.request.url)
    """

    def __init__(self, url: Optional[str] = None, params: Dict[Any, Any] = {}, headers: Dict[Any, Any] = {},
                 cookies: Dict[Any, Any] = {}, timeout: Optional[int] = None, allow_redirects: bool = True,
                 auth: Optional[AuthBase] = None) -> None:
        self.url = url

        self.headers = default_headers()
        self.headers.update(headers)

        self.auth = auth

        self.proxies = {}

        #: Event-handling hooks.
        self.hooks = default_hooks()

        self.params = params
        self.params.update(params)

        self.stream = False
        self.verify = True
        self.cert = None
        self.max_redirects = DEFAULT_REDIRECT_LIMIT
        self.trust_env = True

        self.cookies = cookiejar_from_dict({})
        self.cookies.update(cookies)

        self.adapters = OrderedDict()

        self.mount('https://', HTTPAdapter())

        self.mount('http://', HTTPAdapter())

        self.timeout = timeout
        self.allow_redirects = allow_redirects

    def request(self, method: str, url: str, *args: Any, **kwargs: Any, ) -> requests.models.Response:
        url = f'{self.url}{url}'

        for k in {'params', 'headers', 'cookies'}:
            if k in kwargs:
                # append
                kwargs[k], temp = deepcopy(getattr(self, k)), kwargs[k]
                kwargs[k].update(temp)
            elif getattr(self, k):
                # overwrite
                kwargs[k] = getattr(self, k)

        # overwrite
        for k in {'timeout', 'allow_redirects', 'auth'}:
            if k not in kwargs and self.timeout:
                kwargs[k] = getattr(self, k)

        return super().request(
            method=method,
            url=url,
            *args,
            **kwargs,
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['requests_default']

package_data = \
{'': ['*']}

install_requires = \
['pylint>=2.12.2,<3.0.0', 'requests>=2.16.0,<3.0.0']

setup_kwargs = {
    'name': 'requests-default',
    'version': '0.1.1',
    'description': 'Request for default',
    'long_description': "# requests-default\n## Info\nThis set default `requests <https://docs.python-requests.org/en/latest/>`_'s `url`, `headers` etc...\n\n```python\nfrom requests_base import DefaultSession\n    session = DefaultSession(base_url='https://example.com/resource/', headers={'api-key': 'aabbcc'})\n    r = session.get('sub-resource/', params={'foo': 'bar'})\n    print(r.request.url)\n```\n\n## License\nDistributed under the terms of the MIT license, pytest is free and open source software.",
    'author': 'geonwoo.kim',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ultimatelife/requests-default',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
